# -*- coding: utf-8 -*-
"""
Spyder Editor

This is a temporary script file.
"""

from astropy import units as u
from astropy.units import cds # Centre de Données astronomiques de Strasbourg Standards for Astronomical Catalogues 2.0
from astropy import constants as const
import csv
import datetime
import glob
import matplotlib.pyplot as plt
from matplotlib.ticker import ScalarFormatter
import numpy as np
import os
import pandas as pd
from scipy.integrate import dblquad
from scipy.optimize import curve_fit # Gaussian distirubtion
import sys

u.slm = u.def_unit('standard litre per minute', u.l/u.min)
plt.close("all")

#%% Inputs

doesOutput = False
correctionBkg = 0.06    # The relative error induced by thermal background
symmetryError = 0.04    # Relative errors of signal of interest due to mis-alignment with respect to the heat loads

measMatrix = [
        ['Self-extinguished plasma', '210319-164735_UG', [2820, 2900]],   # 30 Hz, 1.2 slm, 33 Pa, self-stopping at 195 mm
        ]

#%% Constants

atomMass = 1*u.Da
dataFolder = 'Data'
dataFileExten = 'csv'
scanFileExten = 'xlsx'
scanFileGasList = ['H2', 'D2']
scanFileTypeList = ['time', 'flow', 'pose']
#comboFile = '../Cascaded_arc_source/source_combo_database.xlsx'
comboFile = '../Cascaded_arc_source/source_model_database.xlsx'
roomTemp = 20*u.deg_C
ambTemp = 300*u.K   # Ambient temperature. The wall temperature during a hydrogen plasma
gasTemp = 1000*u.K
sensorRadius = 9*u.mm
sensorOffCenter = 13*u.mm
posPlasmaMin = 150  # in mm. The postiion where the Gaussian distribution starts
posMin = 140        # in mm. The minimum of the probe position in the analysis
posMax = 240        # in mm. The maximum of the probe position in the analysis
flowMax = 2.6       # in slm. The maximum of the gas flow in the analysis
heatFluxUnit = u.W/u.m**2
thermalInsulance = 1e-3*u.K/heatFluxUnit
thermalElectricCoeffs = np.array([0, 3.874810e1, 3.329222e-2, 2.061824e-4, -2.188225e-6, 1.099688e-8, -3.081575e-11, 4.547913e-14, -2.751290e-17])  # type T thermocouple at 0 to 400 degC, NIST.monograph 175.page 253
sensorSensitivity = 5.42*u.uV/heatFluxUnit   # HRAD01 (Pt), calibrated by TNO
sensorSenError = 0.15*u.uV/heatFluxUnit         # HRAD01 (Pt), calibrated by TNO
recombEnergy = (436*u.kJ/u.mol)/const.N_A
recombCoeffDiff = 0.14     # Info from ASML report by van der Horst, D000581561/02
recombCoeffError = 0    # 0.105, mainly due to the uncertainty of the radical profile
conversionFactor = 2/recombEnergy/recombCoeffDiff
emissPt = 0.15      # TNO20171222 page 56, 0.05 in Sabuga.2001
emissAlOx = 0.5     # TNO20171222 page 56, 0.90 in Y.S. Touloukian and C.Y. Ho, Thermophysical Properties of Matter (1972)
secondRootParams = np.array([42.34, 10.08])    # Based on Langmuir probe data
ch1Label = 'Pt'
ch2Label = 'AlOx'
timeFormat = "%Y/%m/%d %H:%M:%S"
timeUnit = u.s
keyword1stHeader = 'NO.'
numMeasJump = 20        # the number of data points to average before and after a jump
indexPress = 3          # index of pressure column in the position data file
numSettings = 2
numMeasSkipped = 2
numMeasFitting = 10     # the mininum number of measurement points to fit
keywordUnitLeft = '('
keywordUnitRight = ')'
fractionPeriod = 0.85       # The fraction of time elaspsed before taking a measurement sample for one position
numLangmuirFields = 8   # The number of reserved fields for Langmuir probe data
plotMarkList = ['o', '^', 's', 'D']
plotLineList = ['-', '--', '-.', ':']
plotFigSizeX = 14
plotFigSizeY = 8
plotMarkSize = 12
plotLineWidth = 4
plotFontSize = 20
plotCapSize = 8

#%% Functions

def SeebeckCoeffTypeT(temperatures):
    temperatures = np.atleast_1d(temperatures)
    temperatureArray = u.Quantity(temperatures).to(u.deg_C, equivalencies = u.temperature())     # Quantity converts temperatures into floats, avoiding integers
    SeebeckArray = np.zeros(np.size(temperatureArray))

    ExpoArray = np.arange(np.size(thermalElectricCoeffs))   # Array of the exponents
    for indexTemperature, temperature in enumerate(temperatureArray):
        tempPoly = temperature.value**ExpoArray[:-1]        # The highest order disappears in the derivative calculatoin
        SeebeckArray[indexTemperature] = np.sum(thermalElectricCoeffs[1:]*ExpoArray[1:]*tempPoly)
    
    return SeebeckArray*u.uV/u.K

def Gaussian1D(x, *parameters):
    A, mu, sigma = parameters
    return A*np.exp(-(x-mu)**2/(2.*sigma**2))

def SecondRoot(x, *params): # A fourth-root function fits the data better. However, it lacks of theoretical support and gives a negative intersection.
    slope, intersection = params
    return slope*x**(-1/2) + intersection

def FigureDecoration(ax, xStr, yStr):
    ax.set_xlabel(xStr, fontsize = plotFontSize)
    ax.set_ylabel(yStr, fontsize = plotFontSize)
    ax.xaxis.set_tick_params(labelsize = plotFontSize)
    ax.yaxis.set_tick_params(labelsize = plotFontSize)
    ax.yaxis.get_offset_text().set_size(plotFontSize)    # the exponent is an offset computed by matplotlib
    ax.yaxis.set_major_formatter(ScalarFormatter(useMathText=True))
    if xStr and len(ax.lines) > 1:  # Display the legend only when x string is defined and more than 2 curves are plotted
        ax.legend(fontsize = plotFontSize, loc = 'upper left')    

#%% Readout

numMeas = len(measMatrix)
dataScanList = [None]*numMeas
doesScanList = [False]*numMeas
scanFileSetPeer = ''
rootsFreqList = [None]*numMeas
sigIntereList = [None]*numMeas
bodyTempList = [None]*numMeas
surfTempList = [None]*numMeas
heatFluxList = [None]*numMeas
atomFluxList = [None]*numMeas
dataFitList = [None]*numMeas
sigFitList = [None]*numMeas
GaussianCoeffList = [None]*numMeas
GaussianCovList = [None]*numMeas
gasFlowList = [None]*numMeas
gasPressList = [None]*numMeas
for indexMeas, measProfile in enumerate(measMatrix):
    dataFile = dataFolder + '/' + measProfile[1] + '.' + dataFileExten
    if os.path.isfile(dataFile):
        rawDataList = list(csv.reader(open(dataFile, 'r'), delimiter=','))
        for indexLine, lineStrList in enumerate(rawDataList):
            if lineStrList[0] == keyword1stHeader:
                numHeaderLines = indexLine + 1
                ch1UnitString = lineStrList[2]
                ch2UnitString = lineStrList[3]
                if ch1UnitString != ch2UnitString:
                    sys.exit('The two sensors have different output units.')
                tempUnit = eval('u.' + lineStrList[4][:3] + '_' + lineStrList[4][3])
                break
        
        rawDataMatrix = np.array(rawDataList[numHeaderLines:])
        if '+++++++' in rawDataMatrix:
            print('Out-of-range readout for ' + measMatrix[indexMeas][0])
            break
        initTimeStr = rawDataMatrix[0, 1]
        if measProfile[2]:
            indexStart = measProfile[2][0]
            indexEnd = measProfile[2][1]
        else:
            indexStart = 0
            indexEnd = ((datetime.datetime.strptime(rawDataMatrix[-1, 1], timeFormat) - datetime.datetime.strptime(initTimeStr, timeFormat)).total_seconds()*u.s).to(timeUnit).value

        sensElapsedTimeArray = np.zeros(rawDataMatrix.shape[0])*timeUnit  # Do not use the time stamps, which slows down the data analysis
        for indexSenTime, senTimeStr in enumerate(rawDataMatrix[:,1]):
            sensElapsedTimeArray[indexSenTime] = ((datetime.datetime.strptime(senTimeStr, timeFormat) - datetime.datetime.strptime(initTimeStr, timeFormat)).total_seconds()*u.s).to(timeUnit)
        
        figTime, axTime = plt.subplots(1, figsize=(plotFigSizeX, plotFigSizeY) )
        rawSigPt = rawDataMatrix[:, 2].astype(float)
        rawSigAlOx = rawDataMatrix[:, 3].astype(float)
        rawTemp = rawDataMatrix[:, 4].astype(float)
        rawSigIntere = rawSigPt - rawSigAlOx
#        axTime.plot(sensElapsedTimeArray, rawSigPt, linestyle = plotLineList[2], label = ch1Label)
#        axTime.plot(sensElapsedTimeArray, rawSigAlOx, linestyle = plotLineList[1], label = ch2Label)
        axTime.plot(sensElapsedTimeArray, rawSigIntere, linestyle = plotLineList[0], linewidth = plotLineWidth, label = ch1Label + ' - ' + ch2Label)
        FigureDecoration(axTime, 'Time [' + sensElapsedTimeArray.unit.to_string() + ']', 'Differential signal [' + ch1UnitString + ']')
        axTime.set_xlim(left = indexStart, right = indexEnd)
        axTime.set_ylim(bottom = 0)
#        axTime.set_title(measMatrix[indexMeas][0], fontsize = plotFontSize)

        axTime2 = axTime.twinx()
        axTime2.plot(sensElapsedTimeArray, rawTemp, 'k', linestyle = plotLineList[3], linewidth = plotLineWidth)
        axTime2.set_ylabel('Probe body tempearture [' + tempUnit.to_string('latex') + ']', fontsize = plotFontSize)
        axTime2.yaxis.set_tick_params(labelsize = plotFontSize)

        scanFile = glob.glob(dataFolder + '/' + measProfile[1] + '_*')[0]
        if os.path.isfile(scanFile): # Scan data file exists
            for indexType, scanFileType in enumerate(scanFileTypeList):
                if scanFileType in scanFile:
                    break
            if indexMeas and scanFileType != scanFileSetPeer:
                sys.exit('Different types of scans cannot be analysed together.')
            rootsFreqList[indexMeas] = int(scanFile[scanFile.index('Hz') - 2:scanFile.index('Hz')])
            doesScanList[indexMeas] = True
            scanDataFrame = pd.read_excel(scanFile, sheet_name = 'Sheet1')
            rawScanMatrix = scanDataFrame.values
            indexScanSetting = indexType
            scanHeaderStr = scanDataFrame.columns[indexScanSetting]
            pressHeaderStr = scanDataFrame.columns[indexPress]
            scanElapsedTimeArray = np.zeros(rawScanMatrix.shape[0] - numMeasSkipped)*u.min
            for indexScan, scanTimeObj in enumerate(rawScanMatrix[numMeasSkipped:,0]):
                scanElapsedTimeArray[indexScan] = ((datetime.datetime.strptime(scanTimeObj.strftime(timeFormat), timeFormat) - datetime.datetime.strptime(initTimeStr, timeFormat)).total_seconds()*u.s).to(timeUnit)
            samplingTimeArray = ( scanElapsedTimeArray[1:]*fractionPeriod + scanElapsedTimeArray[:-1]*(1 - fractionPeriod) ).astype(float)
            indicesSampling = np.searchsorted(sensElapsedTimeArray, samplingTimeArray)
            
            rawSigIntereAfterMeas = rawSigIntere[indicesSampling[-1]:]
            indexOff = indicesSampling[-1] + np.argmax(np.diff(rawSigIntereAfterMeas))
            indexJump = indicesSampling[-1] + np.argmin(np.diff(rawSigIntereAfterMeas))
            rawSigIntereAfterJump = rawSigIntere[indexJump:]
            indexSettle = indexJump + np.squeeze(np.where(np.diff(rawSigIntereAfterJump) == 0))[0]
            axTime.plot(sensElapsedTimeArray[indexOff - numMeasJump: indexOff], rawSigIntere[indexOff - numMeasJump: indexOff])                # Highlight the signals to average before the extinguishment
            axTime.plot(sensElapsedTimeArray[indexSettle: indexSettle + numMeasJump], rawSigIntere[indexSettle: indexSettle + numMeasJump])    # Highlight the signals to average after the extinguishment
            
            dataScanList[indexMeas] = np.vstack( (rawScanMatrix[numMeasSkipped:-1, indexScanSetting], rawScanMatrix[numMeasSkipped:-1, indexPress], rawDataMatrix[indicesSampling, 2:5].T ) ).T    # scan setting and hydrogen signals, first stacking and then transposing
            np.place(dataScanList[indexMeas], dataScanList[indexMeas] == '+++++++', '0')
            dataScanList[indexMeas] = np.delete(dataScanList[indexMeas].astype(float), np.where(dataScanList[indexMeas][:, 0] < (flowMax if scanFileType == 'flow' else posMin)), axis = 0)
            dataScanList[indexMeas] = np.delete(dataScanList[indexMeas].astype(float), np.where(dataScanList[indexMeas][:, 0] > (flowMax if scanFileType == 'flow' else posMax)), axis = 0)
            sigPtArray = u.Quantity(dataScanList[indexMeas][:, 2], eval('u.' + ch1UnitString))
            sigAlOxArray = u.Quantity(dataScanList[indexMeas][:, 3], eval('u.' + ch2UnitString))
            sigIntereList[indexMeas] = (sigPtArray - sigAlOxArray)*(1 - correctionBkg)

            bodyTempList[indexMeas] = u.Quantity(dataScanList[indexMeas][:, 4], tempUnit)
            surfTempList[indexMeas] = ( (np.tile(bodyTempList[indexMeas], (2, 1)) ).to(u.K, equivalencies = u.temperature()) + thermalInsulance*u.Quantity( [sigPtArray, sigAlOxArray])/sensorSensitivity).T   # The function 'Quantity' preserves the unit of the arrays
            sensorSenUpperArray = sensorSensitivity*SeebeckCoeffTypeT(bodyTempList[indexMeas])/SeebeckCoeffTypeT(roomTemp)
            gasPressList[indexMeas] = dataScanList[indexMeas][:, 1]*eval('u.' + pressHeaderStr[pressHeaderStr.find(keywordUnitLeft)+1 : pressHeaderStr.find(keywordUnitRight)])

            sigmaArray = SecondRoot(gasPressList[indexMeas].value, *secondRootParams)*u.mm/2/np.sqrt(2*np.log(2))
            sensorArea = np.pi*sensorRadius**2
            ratioSensor2centerArray = np.zeros(len(sigmaArray))
            for indexSigma, sigmaValue in enumerate(sigmaArray.si.value):
                preFactorGaussianValue = 1/2/np.pi/sigmaValue**2
                ratioSensor2centerArray[indexSigma] = (dblquad(lambda x, y: 2*preFactorGaussianValue*np.exp(-(x**2 + y**2)/2/sigmaValue**2), sensorOffCenter.si.value - sensorRadius.si.value, sensorOffCenter.si.value + sensorRadius.si.value, lambda x:0, lambda x:np.sqrt(sensorRadius.si.value**2 - (x - sensorOffCenter.si.value)**2 ))/sensorArea.si.value/preFactorGaussianValue)[0]
            
#            heatFluxList[indexMeas] = u.Quantity( [ sigIntereList[indexMeas]/sensorSenUpperArray, const.sigma_sb*(emissPt*(ambTemp**4 - surfTempList[indexMeas][:,0]**4) - emissAlOx*(ambTemp**4 - surfTempList[indexMeas][:,1]**4)), sigIntereList[indexMeas]*(1/(sensorSensitivity - sensorSenError) - 1/sensorSensitivity) ] ).T
            heatFluxList[indexMeas] = (u.Quantity( [ sigIntereList[indexMeas]/sensorSensitivity, sigIntereList[indexMeas]/sensorSensitivity*np.sqrt( symmetryError**2 + (sensorSenError**2 + (sensorSenUpperArray - sensorSensitivity)**2)/sensorSensitivity**2 ), sigIntereList[indexMeas]/sensorSensitivity*np.sqrt( symmetryError**2 + (sensorSenError/sensorSensitivity)**2 + (1 - ratioSensor2centerArray)**2 ) ] ).T).to(heatFluxUnit)
            atomFluxList[indexMeas] = (heatFluxList[indexMeas]*conversionFactor).si
            atomFluxList[indexMeas][:, 1:3] = np.sqrt( atomFluxList[indexMeas][:, 1:3]**2 + (recombCoeffError/recombCoeffDiff*np.tile(atomFluxList[indexMeas][:, 0][:, None], 2 ))**2 )
            scanFileSetPeer = scanFileType

#%% Visualization

if any(doesScanList):
    figScan, axScan = plt.subplots(1, figsize=(plotFigSizeX, plotFigSizeY) )
#    figScan, axScan = plt.subplots(2, sharex = True, figsize=(plotFigSizeX, plotFigSizeY) )
    figHeat, axHeat = plt.subplots(1, figsize=(plotFigSizeX, plotFigSizeY) )
    figFlux, axFlux = plt.subplots(1, figsize=(plotFigSizeX, plotFigSizeY) )

if doesOutput:
    writer = pd.ExcelWriter(comboFile, engine = 'xlsxwriter')
for indexMeas, measProfile in enumerate(measMatrix):
    if doesScanList[indexMeas]:
        print(measMatrix[indexMeas][0])
        lineObj = axScan.plot(dataScanList[indexMeas][:,0], sigIntereList[indexMeas], marker = plotMarkList[indexMeas%len(plotMarkList)], markersize = plotMarkSize, linestyle = 'None', label = measMatrix[indexMeas][0])
#        lineObj = axScan[0].plot(dataScanList[indexMeas][:,0], sigIntereList[indexMeas], marker = plotMarkList[indexMeas%len(plotMarkList)], markersize = plotMarkSize, linestyle = 'None', label = measMatrix[indexMeas][0])
#        axScan[1].plot(dataScanList[indexMeas][:,0], bodyTempList[indexMeas], marker = plotMarkList[indexMeas%len(plotMarkList)], markersize = plotMarkSize, linestyle = 'None', label = measMatrix[indexMeas][0])
        axHeat.errorbar(dataScanList[indexMeas][:,0], heatFluxList[indexMeas][:,0].value, yerr = heatFluxList[indexMeas][:,1:3].T.value, fmt = plotMarkList[indexMeas], markersize = plotMarkSize, capsize = plotCapSize, label = measMatrix[indexMeas][0])    # errorbar only accepts positive values
        axFlux.errorbar(dataScanList[indexMeas][:,0], atomFluxList[indexMeas][:,0].value, yerr = atomFluxList[indexMeas][:,1:3].T.value, fmt = plotMarkList[indexMeas%len(plotMarkList)], markersize = plotMarkSize, capsize = plotCapSize, label = measMatrix[indexMeas][0])    # errorbar only accepts positive values
        if scanFileType == 'flow':
            gasDensityArray = (gasPressList[indexMeas]/cds.k/gasTemp).si
            dissDegreeArray = (atomFluxList[indexMeas][:, 0]/(1/4*np.sqrt(8*cds.k*gasTemp/np.pi/atomMass))/gasDensityArray).si
            if doesOutput:
                outputDataFrame = pd.DataFrame(np.hstack( ( np.zeros( (dataScanList[indexMeas].shape[0], numLangmuirFields) ), dissDegreeArray[:, None].value, atomFluxList[indexMeas].value) ), index = dataScanList[indexMeas][:, 0],
                                               columns = ['Pressure (mbar)', 'Te', 'Te lower error', 'Te upper error', 'Ionization degree', 'Ion flux', 'Ion flux lower error', 'Ion flux upper error', 'Dissociation degree', 'Atom flux', 'Atom flux lower error', 'Atom flux upper error'])
                outputDataFrame.to_excel(writer, sheet_name = str(rootsFreqList[indexMeas]) + 'Hz')
        elif scanFileType == 'pose':
            indexFlow = scanFileTypeList.index('flow')
            flowHeaderStr = scanDataFrame.columns[indexFlow]
            gasFlowList[indexMeas] = list(set(rawScanMatrix[:, indexFlow].astype(float)))[0]*eval('u.' + flowHeaderStr[flowHeaderStr.find(keywordUnitLeft)+1 : flowHeaderStr.find(keywordUnitRight)])
            dataFitList[indexMeas] = np.delete(dataScanList[indexMeas], np.where(dataScanList[indexMeas][:,0] < posPlasmaMin), axis = 0)
            sigFit = (dataFitList[indexMeas][:, 2] - dataFitList[indexMeas][:, 3])*(1 - correctionBkg)
            if dataFitList[indexMeas].shape[0] > numMeasFitting:    # Enough data points to fit
                indexPosMaxFlux = np.argmax( sigFit )
                params0 = [ np.amax(sigFit), dataFitList[indexMeas][indexPosMaxFlux, 0], dataFitList[indexMeas][indexPosMaxFlux, 0]/2 ]    # the initial guess for the fitting coefficients (A, mu and sigma)
                GaussianCoeffList[indexMeas], GaussianCovList[indexMeas] = curve_fit(Gaussian1D, dataFitList[indexMeas][:,0], sigFit, p0 = params0, bounds = (0, np.inf))

                print('Peak of the Gassian distribution is ' + "{:.2f} ".format(GaussianCoeffList[indexMeas][0]) + "± {:.2f} ".format(np.sqrt(np.diag(GaussianCovList[indexMeas])[0])) + ch1UnitString)
                factorFWHM = 2*np.sqrt(2*np.log(2))
                print('FWHM of the Gassian distribution is ' + "{:.2f} ".format(factorFWHM*GaussianCoeffList[indexMeas][2]) + "± {:.2f} ".format(factorFWHM*np.sqrt(np.diag(GaussianCovList[indexMeas])[2])) + scanHeaderStr[scanHeaderStr.find(keywordUnitLeft)+1 : scanHeaderStr.find(keywordUnitRight)])
                posArrayLinear = np.linspace(np.amin(dataFitList[indexMeas][:,0]),np.amax(dataFitList[indexMeas][:,0]), int(np.amax(dataFitList[indexMeas][:,0])-np.amin(dataFitList[indexMeas][:,0])))
                sigIntereFitArray = Gaussian1D(posArrayLinear, *GaussianCoeffList[indexMeas])*sigIntereList[indexMeas].unit
                atomFluxFitArray = sigIntereFitArray/sensorSensitivity*conversionFactor   # Get the fitting result
                axScan.plot(posArrayLinear, sigIntereFitArray, linestyle = plotLineList[indexMeas%len(plotLineList)], color = lineObj[-1].get_color(), label = 'Fit for ' + measMatrix[indexMeas][0])
#                axScan[0].plot(posArrayLinear, sigIntereFitArray, linestyle = plotLineList[indexMeas%len(plotLineList)], color = lineObj[-1].get_color(), label = 'Fit for ' + measMatrix[indexMeas][0])
                axFlux.plot(posArrayLinear, atomFluxFitArray, linestyle = plotLineList[indexMeas%len(plotLineList)], color = lineObj[-1].get_color(), label = 'Fit for ' + measMatrix[indexMeas][0])
                measuredRadicalFlux = (GaussianCoeffList[indexMeas][0]*eval('u.' + ch1UnitString)/sensorSensitivity*conversionFactor).si
                print('Measured atom flux {:.2g}'.format(measuredRadicalFlux))
                measuredRadicalDensity = (measuredRadicalFlux/(1/4*np.sqrt(8*cds.k*gasTemp/np.pi/atomMass))).si
                print('Measured atom density {:.2g}'.format(measuredRadicalDensity))
                print('Measured dissociation degree {:.2f}%'.format(1e2*(measuredRadicalDensity/(gasPressList[indexMeas][indexPosMaxFlux]/cds.k/gasTemp)).si))
    else:
        print('No scan files')

if 'writer' in locals():
    writer.save()
    writer.close()
if any(doesScanList):
    indexMeasScan = doesScanList.index(True)
    FigureDecoration(axScan, scanHeaderStr, 'Differential signal (' + ch1UnitString + ')')
#    FigureDecoration(axScan[0], '', 'Differential signal (' + ch1UnitString + ')')
#    if len(set(gasFlowList)) == 1:  # Equal gas flows
#        axScan.set_title(gasFlowList[indexMeasScan], fontsize = plotFontSize)
#    FigureDecoration(axScan[1], scanHeaderStr, 'Temperature (' + bodyTempList[indexMeasScan].unit.to_string('latex') + ')')
    FigureDecoration(axScan, scanHeaderStr, 'Temperature (' + bodyTempList[indexMeasScan].unit.to_string('latex') + ')')
    FigureDecoration(axHeat, scanHeaderStr, 'Recombination Heat Flux (' + heatFluxList[indexMeasScan].unit.to_string('latex') + ')')
    FigureDecoration(axFlux, scanHeaderStr, 'Atom Flux (' + atomFluxList[indexMeasScan].unit.to_string('latex') + ')')
