# -*- coding: utf-8 -*-
"""
Created on Fri Sep 22 13:40:13 2017

@author: scwang
"""

from astropy import units as u
from astropy.units import cds # Centre de Données astronomiques de Strasbourg Standards for Astronomical Catalogues 2.0
import csv
import math
import matplotlib.pyplot as plt
from matplotlib.ticker import ScalarFormatter
import numpy as np
import os
import pandas as pd
import re # regular expression
import sys
from scipy.optimize import curve_fit # Gaussian distirubtion
plt.close("all")
u.slm = u.def_unit('slm', u.bar*u.l/u.min)

#%% Inputs

doesOutput = False
dataList = [
#        ['25 Hz', '20210510_cleaned_D2_H2\H2_0.5mm_7.0mm_4_25Hz_flow'],
#        ['30 Hz', '20210510_cleaned_D2_H2\H2_0.5mm_7.0mm_6_30Hz_flow'],
#        ['35 Hz', '20210510_cleaned_D2_H2\H2_0.5mm_7.0mm_7_35Hz_flow'],
#        ['42 Hz', '20210510_cleaned_D2_H2\H2_0.5mm_7.0mm_8_42Hz_flow'],
#        ['50 Hz', '20210510_cleaned_D2_H2\H2_0.5mm_7.0mm_9_50Hz_flow'],
        ]

indexInterest = 8
paramStrList = ['Gas flow', 'Position', 'Source current', 'Source voltage', 'Pressure', 'Gas density',
        # index   0             1            2                  3             4           5
                  'Te', 'Ne', 'Ion flux','Heat flux', 'Sheath potential', 'Sheath thickness']
        # index   6,     7      8            9                  10            11

#%% Constants
numSettings = 2         # Setting parameters: flow and position (or bias)
numParamPerGroup = 6    # Parameters in 3 groups: inputs, outputs, errors
numParamTotal = 3*numParamPerGroup
filenameLength = 20     # yymmdd_hhmmss_RM.dat
#comboFile = '../Cascaded_arc_source/source_combo_database.xlsx'
comboFile = '../Cascaded_arc_source/source_model_database.xlsx'
numLangmuirFields = 8   # The number of reserved fields for Langmuir probe data

maxGasFlow = 4.0        # in slm, for hydrogen plasmas of interest
minGasFlow = 0.9        # in slm, for hydrogen plasmas of interest
minProbePosition = 100  # in mm, for hydrogen plasmas of interest
maxProbePosition = 180  # in mm, for hydrogen plasmas of interest
minGaussPosition = 110  # in mm, for Gaussian profiles
maxGaussPosition = 170  # in mm, for Gaussian profiles
maxSourceCurrent = 45   # in A, for hydrogen plasmas of interest
minSourceCurrent = 35   # in A, for hydrogen plasmas of interest
centerVoltagePositive = 0.2
centerVoltageNegative = -0.2

voltageUnit = u.V
currentUnit = u.A
IVcurveUnit = u.uA

flowUnit = u.slm
pressureUnit = cds.mbar
lengthUnit = u.mm

energyUnit = u.eV
densityUnit = u.m**-3
fluxUnit = u.m**-2*u.s**-1
heatUnit = u.W*u.m**-2
potentialUnit = u.V
thicknessUnit = u.mm

keywordExtraList = ['mm', '-']
indexShiftExtraList = [-3, 0]
unitExtraList = [lengthUnit, voltageUnit]
indexLengthExtra = 3

gasTemp = 1000*u.K              # 1000 K stated in Vankan 2004 PhD thesis, atom perpendicular temperature
crossSectionNN = 1E-15*u.cm**2  # neutral-neuteral collision, Elg PhD thesis (2016)
adiabaticIndex = 1.3        # quasi-adiabatic flow, Mazouffre.2001.PhD thesis
effectBohmCoeffPlanar = 0.5       # effective Bohm coefficient in Chen.2002 and Chen.2003 (lecture notes) page 7. For Ti=0, see Lieberman Eq.6.2.13 in page 209
DebyeLengthGuess = 27*u.um    # ~33 um and ~21 um respectively for 1.5 slm and 2.5 slm with geometrical corrections
effectBohmCoeffRelaError = 0.10  # Estimated based on the uncertainties of the Debye length
heatTransCoeff = 8          # Stangeby Eq. 2.95
secondaryEmissionCoeff = 0
areaRelaError = 0.05       # two error sources: 1. iteration process 2. relative error of the area of the W wires based on the difference between saturation currents at +10 and -10 V
plotFigSizeX = 10
plotFigSizeY = 6
plotFontSize = 16
plotMarkSize = 12
plotCapSize = 8

#%% Functions

def ActualPressure(gas,x):  # MKS 970B MicroPirani
    if gas=='H2' or gas=='D2':
        R0 = 157.77
        order = -0.222
        Tr = 5.464
        order2 = 0.2157
    else:
        R0 = 1
        order = 0
        Tr = 0
        order2 = 0
    return R0*x**order*np.exp(-Tr/x**order2)

gasDict = {'H2': {'name':'Hydrogen', 'mass': 1.008*u.Da, 'radius': 1.61*u.angstrom, 'dissociation': 4.50*u.eV, 'ionization': 13.60*u.eV, 'number of atoms': 3},   # Batsanov.2001, average of radii in two directions. Most of ions are H3+ based on Hansen.2012
           'D2': {'name':'Deuterium','mass': 2.014*u.Da, 'radius': 1.61*u.angstrom, 'dissociation': 4.56*u.eV, 'ionization': 15.46*u.eV, 'number of atoms': 3},   # Liu.2010. Most of ions are D3+ based on Hansen.2012
           'He': {'name':'Helium',   'mass': 4.003*u.Da, 'radius': 1.40*u.angstrom, 'dissociation': 0.00*u.eV, 'ionization': 24.59*u.eV, 'number of atoms': 1},
           'Ar': {'name':'Argon',    'mass':39.948*u.Da, 'radius': 1.88*u.angstrom, 'dissociation': 0.00*u.eV, 'ionization': 15.76*u.eV, 'number of atoms': 1},
           }

def effectiveBohmCoefficientRatio(xi):
    return (0.607 + 2432/np.exp(7.01*xi**0.096))/0.607  # Eq.29 in Chen.2002 for argon plasmas

def Gaussian(x, *params):
    A, mu, sigma = params
    return A*np.exp(-(x-mu)**2/(2.*sigma**2))

def SecondRoot(x, *params):     # A fourth-root function fits the data better. However, it lacks of theoretical support and gives a negative intersection.
    slope, intersection = params
    return slope*x**(-1/2) + intersection

def ParameterExtraction(directory, file, doesPlot):
    # Data readout
    KeithleyString = open(directory + '/' + file,'r').read().replace('\\','/')  # Replace backslashes with slahes
    filenameEndIndex = next(i for i,x in enumerate(KeithleyString) if x=='>')
    dataFile = directory + '/' + KeithleyString[filenameEndIndex-filenameLength:filenameEndIndex]
    
    rawDataMatrix = np.array(list(csv.reader(open(dataFile, 'r'), delimiter='\t'))).astype(float)

    centerIndexZero = next(i for i,x in enumerate(rawDataMatrix[:,2]) if float(x)< 0)
    centerVoltageZero = rawDataMatrix[centerIndexZero,1]
    centerIndexPositive = next(i for i,x in enumerate(rawDataMatrix[:,1]) if float(x)<= centerVoltageZero + centerVoltagePositive)
    centerIndexNegative = next(i for i,x in enumerate(rawDataMatrix[:,1]) if float(x)<= centerVoltageZero + centerVoltageNegative)
    rawDataMatrix[:,2]=(rawDataMatrix[:,2]*currentUnit).to(IVcurveUnit)
    
    # fit linear functions to the data
    voltageArrayCenter = rawDataMatrix[centerIndexPositive:centerIndexNegative + 1, 1]
    centerLineCoeffs, centerLineVarMatrix = np.polyfit(voltageArrayCenter, rawDataMatrix[centerIndexPositive:centerIndexNegative + 1, 2],1, cov = True)
    centerLineFun = np.poly1d(centerLineCoeffs)

    edgeIndexPositive = next(i for i,x in enumerate(rawDataMatrix[:,1]) if float(x)<=np.max(rawDataMatrix[:,1])/2)
    edgeIndexNegative = next(i for i,x in enumerate(rawDataMatrix[:,1]) if float(x)<=np.min(rawDataMatrix[:,1])/2)
    voltageArrayEdgePositive = rawDataMatrix[0:edgeIndexPositive, 1]
    edgeLinePositiveCoeffs,edgeLinePositiveVarMatrix = np.polyfit(voltageArrayEdgePositive, rawDataMatrix[0:edgeIndexPositive, 2], 1, cov=True)
    edgeLinePositiveFun = np.poly1d(edgeLinePositiveCoeffs)
    voltageArrayEdgeNegative = rawDataMatrix[edgeIndexNegative:, 1]
    edgeLineNegativeCoeffs,edgeLineNegativeVarMatrix = np.polyfit(voltageArrayEdgeNegative, rawDataMatrix[edgeIndexNegative:, 2], 1, cov=True)
    edgeLineNegativeFun = np.poly1d(edgeLineNegativeCoeffs)
    
    # Calculation
    intersectPositive = edgeLinePositiveFun(centerVoltageZero)
    intersectNegative = edgeLineNegativeFun(centerVoltageZero)    
    saturationCurrentGuess = (intersectPositive - intersectNegative)*IVcurveUnit/2
    saturationCurrentRelaError = np.sqrt( areaRelaError**2 + effectBohmCoeffRelaError**2 + (edgeLinePositiveVarMatrix[1,1]/edgeLinePositiveCoeffs[1])**2 + (edgeLineNegativeVarMatrix[1,1]/edgeLineNegativeCoeffs[1])**2 )
    ionMass = gasDict.get(file[0:2],{}).get('mass',-1)*gasDict.get(file[0:2],{}).get('number of atoms',-1)     # returns negative mass if nothing matches
    probeArea = math.pi/4*probeDiameter**2 + math.pi*probeDiameter*probeLength
    electronTemperatureGuess = (cds.e*saturationCurrentGuess/2/(centerLineCoeffs[0]*IVcurveUnit/voltageUnit)).to(energyUnit)      # Brussaard.1999, Naz.2011
    largePotentialPositive = centerVoltageZero + 10.8*electronTemperatureGuess.value     # Brussaard.1999, page 26
    largePotentialNegative = centerVoltageZero - 10.8*electronTemperatureGuess.value     # Brussaard.1999, page 26
    saturationCurrent = (edgeLinePositiveFun(largePotentialPositive) - edgeLineNegativeFun(largePotentialNegative))*IVcurveUnit/2   # Brussaard.1999, page 26
    electronTemperature = (cds.e*saturationCurrent/2/(centerLineCoeffs[0]*IVcurveUnit/voltageUnit)).to(energyUnit)      # Brussaard.1999, Naz.2011
    electronTemperatureError = (cds.e*saturationCurrent*saturationCurrentRelaError/2/((centerLineCoeffs[0] - centerLineVarMatrix[0,0])*IVcurveUnit/voltageUnit)).to(energyUnit)      # Brussaard.1999, Naz.2011
    ionTemperature = electronTemperature    # Brussaard.1996
    if electronTemperature > 0:
        soundSpeed = (((electronTemperature + adiabaticIndex*ionTemperature)/ionMass)**0.5).to(u.m/u.s)     # Stangeby Eq. 2.55
        effectBohmCoeffCylinderGuess = effectBohmCoeffPlanar*effectiveBohmCoefficientRatio(probeDiameter/2/DebyeLengthGuess)
        electronDensityGuess = (saturationCurrent/effectBohmCoeffCylinderGuess/cds.e/probeArea/soundSpeed).to(densityUnit)    # Naz.2011
        DebyeLength = np.sqrt(cds.eps0*electronTemperature/electronDensityGuess/cds.e**2)
        effectBohmCoeffCylinder = effectBohmCoeffPlanar*effectiveBohmCoefficientRatio(probeDiameter/2/DebyeLength)
        electronDensity = (saturationCurrent/effectBohmCoeffCylinder/cds.e/probeArea/soundSpeed).to(densityUnit)    # Naz.2011
        ionFlux = (effectBohmCoeffPlanar*electronDensity*soundSpeed).to(fluxUnit)        # Stangeby Eq. 2.56 with the factor 2 replaced by the effective Bohm coefficient
        heatFlux = ((heatTransCoeff*electronTemperature + gasDict.get(file[0:2],{}).get('ionization', -1))*ionFlux).to(heatUnit) # Stangeby Eq. 2.94 plus ionization energy
        electronDensityError = electronDensity*saturationCurrentRelaError
        ionFluxError = ionFlux*saturationCurrentRelaError
        heatFluxError = heatFlux*saturationCurrentRelaError
        sheathPotential = (electronTemperature/cds.e/2*np.log(2*np.pi*cds.me/ionMass*(1 + ionTemperature/electronTemperature)*(1 - secondaryEmissionCoeff)**(-2))).to(potentialUnit)  # Stangeby Eq. 25.31, the pre-sheath potential drop (ln2*k*Te/e) is NOT included
        sheathThickness = (np.sqrt(2)/3*np.sqrt(cds.eps0*electronTemperature/electronDensity/cds.e**2)*(2*cds.e*(-sheathPotential)/electronTemperature)**0.75).to(thicknessUnit) # Child-Langmuir law, Lieberman page 214
    else:
        sys.exit('Negative Te. Check the IV curve.')

    if doesPlot:        # Plot the IV curve
        print('{0:.2f}\n{1:.2g}\n{2:.2g}\n{3:.2g}\n{4:.2f}\n{5:.2f}'.format(electronTemperature,electronDensity,ionFlux,heatFlux,sheathPotential,sheathThickness))
        figIV = plt.figure(figsize=(plotFigSizeX, plotFigSizeY))
        axIV = figIV.add_subplot(111)
        axIV.spines['left'].set_position(('data', 0.0))   # Move left y-axis and bottim x-axis to centre of the data
        axIV.spines['bottom'].set_position(('data', 0.0))
        axIV.spines['right'].set_color('none')    # Eliminate upper and right axMeanes
        axIV.spines['top'].set_color('none')
        
        axIV.plot(rawDataMatrix[:,1], rawDataMatrix[:,2], '*')
        axIV.plot(voltageArrayEdgePositive, edgeLinePositiveFun(voltageArrayEdgePositive))
        axIV.plot(voltageArrayEdgeNegative, edgeLineNegativeFun(voltageArrayEdgeNegative))
        axIV.plot(voltageArrayCenter, centerLineFun(voltageArrayCenter))
        
        plt.xlabel('voltage [V]', fontsize = plotFontSize, position = (1,0))
        plt.ylabel('current ['+ IVcurveUnit.to_string('latex') + ']',fontsize = plotFontSize, position=(0,1), rotation=0)
    
    return electronTemperature.value, electronDensity.value, ionFlux.value, heatFlux.value, sheathPotential.value, sheathThickness.value, electronTemperatureError.value, electronDensityError.value, ionFluxError.value, heatFluxError.value, edgeLinePositiveVarMatrix[1,1], edgeLineNegativeVarMatrix[1,1]

def FigureDecoration(ax, xStr, yStr, tStr):
    ax.set_xlabel(xStr, fontsize = plotFontSize)
    ax.set_ylabel(yStr, fontsize = plotFontSize)
    ax.set_title(tStr, fontsize = plotFontSize)
    ax.xaxis.set_tick_params(which = 'both', labelsize = plotFontSize)
    ax.yaxis.set_tick_params(which = 'both', labelsize = plotFontSize)
    formatter = ScalarFormatter(useMathText = True)
    formatter.set_scientific(True)  # This line is needed to show the offset of both the major and minor ticks
    ax.yaxis.set_major_formatter(formatter)
#    ax.yaxis.set_minor_formatter(formatter)
    ax.yaxis.get_offset_text().set_size(plotFontSize)    # the exponent is an offset computed by matplotlib
    if xStr and len(dataList) > 1:
        ax.legend(fontsize = plotFontSize, loc = 'upper left')

#%% Read data from all files

workDir = os.getcwd()   # The working directory is changed to the directory of the script once it is executed
numMeas = len(dataList)
rootsFreqList = [None]*numMeas
averDataList = [None]*numMeas
presArray = np.zeros(numMeas)
widthArray = np.zeros(numMeas)
Data4Combo = [None]*numMeas

for indexMeas, measInfo in enumerate(dataList):
    dataDirectory = (workDir + '\\' + measInfo[1]).replace('\\', '/')
    indicesSlash = [slash.start() for slash in re.finditer('/', dataDirectory)]
    dataFolder = dataDirectory[indicesSlash[-1] + 1:]
    probeDiameter = float(dataFolder[3:6])*eval('u.' + dataFolder[6:8])
    probeLength = float(dataFolder[9:12])*eval('u.' + dataFolder[12:14])
    
    rootsFreqList[indexMeas] = int(dataFolder[dataFolder.index('Hz') - 2:dataFolder.index('Hz')]) if dataFolder.find('Hz') > 0 else 'NA'
    dataMatrix = np.empty([0, 3*numParamPerGroup])     # to store settings and plasma parameters
    isSingleMeasurement = ( len(os.listdir(dataDirectory)) == 2 ) # If there exists only one data point, show the I-V curve
    for file in os.listdir(dataDirectory):
        if not file.endswith('dat'):
            settingFile = file
            gasFlow = float(file[file.index(flowUnit.to_string()) - 3:file.index(flowUnit.to_string())])
            sourceCurrent = float(file[10:14])
            sourceVoltage = float(file[16:21])
            for indexKeywordExtra, keywordExtra in enumerate(keywordExtraList):
                indexExtra = [keywordFile.start() for keywordFile in re.finditer(keywordExtra, file)]      # Search the filename for 'mm'
                if indexExtra:  # the keyword of the extra parameter is found
                    indexKeywordExtraFound = indexKeywordExtra
                    extraParam = float(file[indexExtra[0] + indexShiftExtraList[indexKeywordExtraFound]: indexExtra[0] + indexShiftExtraList[indexKeywordExtraFound] + indexLengthExtra])
                    break
                else:
                    indexKeywordExtraFound = 1  # bias voltage
                    extraParam = 0              # floating
            
            if gasFlow <= maxGasFlow and gasFlow >= minGasFlow and (indexKeywordExtraFound == 0 and extraParam >= minProbePosition and extraParam <= maxProbePosition) and sourceCurrent <= maxSourceCurrent and sourceCurrent >= minSourceCurrent:    # Take measurements under normal condition. Note the file extention is excluded.
                presReadout = float(file[file.index(pressureUnit.to_string()) - 5:file.index(pressureUnit.to_string())])
                pressure = presReadout if 'Baratron' in file else ActualPressure(file[0:2], presReadout)    # Baratron gives absolute pressure readout. Otherwise conversion is needed.
                dataMatrix = np.concatenate((dataMatrix, np.array([[gasFlow, extraParam, sourceCurrent, sourceVoltage, pressure, ((pressure*pressureUnit/cds.k/gasTemp).to(densityUnit)).value] + list(ParameterExtraction(dataDirectory,file, isSingleMeasurement))])), axis = 0)
                if dataMatrix[-1, numParamPerGroup] < 0:   # Negative electron temperature
                    dataMatrix = np.delete(dataMatrix, -1, axis = 0)
                    print('Exclude ' + file)
    
    if len(dataMatrix) == 0:    # No data was read
        sys.exit('No data points come in. Check the setting limitations.')
    
    unitArray = np.array([flowUnit, unitExtraList[indexKeywordExtraFound], currentUnit, voltageUnit, pressureUnit, densityUnit,  # Experimental inputs/settings
                          energyUnit, densityUnit, fluxUnit, heatUnit, potentialUnit, thicknessUnit])                               # Experimental outputs/parameters

#%% Visualization of the fluctuation before averaging

    for indexSetting in range(numSettings):
        if len(dataMatrix[:, indexSetting]) > 1:
            if len(set(dataMatrix[:, indexSetting])) == 1:  # Only one data point for this setting
                if 'indexConstant' not in locals():
                    indexConstant = indexSetting
                else:
                    if indexConstant != indexSetting:
                        sys.exit('Different types of scans cannot be analysed together.')
            else:
                indexVariable = indexSetting
        else:
            indexConstant = 0
            indexVariable = 0
        
    figRaw, axRaw = plt.subplots(3, sharex = True, figsize = (plotFigSizeX, plotFigSizeY))
    axRaw[0].plot(dataMatrix[:,indexVariable], dataMatrix[:,-2],'r*')
    axRaw[0].set_ylabel('edgeLinePositiveVariance', fontsize = plotFontSize)
    axRaw[0].set_title('Fitting Parameters (before averaging)', fontsize = plotFontSize)
    
    axRaw[1].plot(dataMatrix[:,indexVariable], dataMatrix[:,-1], '*')
    axRaw[1].set_ylabel('edgeLineNegativeVariance', fontsize = plotFontSize)
    
    axRaw[2].plot(dataMatrix[:,indexVariable], dataMatrix[:,-6], '*')
    axRaw[2].set_xlabel('Bias voltage' if indexKeywordExtraFound == 1 else paramStrList[indexVariable] + ' (' + unitArray[indexVariable].to_string('latex') + ')', fontsize = plotFontSize)
    axRaw[2].set_ylabel('Te error', fontsize = plotFontSize)

#%% Take averages

    indiDataMatrix = np.empty([0, numParamTotal]) # to store inputs and individual plasma outputs
    averDataList[indexMeas] = np.empty([0, numParamTotal]) # to store inputs and mean plasma outputs
    while(dataMatrix.shape[0] > 0):
        indicesRepro = []
        settingsChosen = dataMatrix[0, 0:numSettings]
        for indexDataMatrix in range(dataMatrix.shape[0]):
            if (dataMatrix[indexDataMatrix, 0:numSettings] == settingsChosen).all():
                indicesRepro.append(indexDataMatrix)

        averDataList[indexMeas] = np.append(averDataList[indexMeas], np.array([np.append(settingsChosen, np.sum(dataMatrix[indicesRepro, numSettings:numParamTotal], axis = 0)/len(indicesRepro))]), axis = 0)
        indiDataMatrix = np.vstack((indiDataMatrix, dataMatrix[indicesRepro, :]))
        dataMatrix = np.delete(dataMatrix, indicesRepro, axis = 0)  # delete the entire row
    
    averDataList[indexMeas] = averDataList[indexMeas][np.argsort(averDataList[indexMeas][:, indexVariable]),:]  # Sort the variable setting

#%% Visualization after averaging

if doesOutput:
    Data4Combo = [None]*numMeas
    for indexFreq, rootsFreq in enumerate(rootsFreqList):
        comboDataFrame = pd.read_excel(comboFile, sheet_name = str(rootsFreq) + 'Hz')
        Data4Combo[indexFreq] = [ rootsFreq, comboDataFrame.values ]
    comboHeaders = comboDataFrame.columns   # This command modifies repeated headers, e.g. Pressure --> Pressure.1
    writer = pd.ExcelWriter(comboFile, engine = 'xlsxwriter')

figMean, axMean = plt.subplots(2, sharex = True, figsize=(plotFigSizeX, plotFigSizeY))
for indexMeas, measInfo in enumerate(dataList):
    print(measInfo[0])  # The scan label
    x0=averDataList[indexMeas][:,indexVariable]
    y2=averDataList[indexMeas][:,indexInterest]
    
    axMean[0].errorbar(x0, averDataList[indexMeas][:,numParamPerGroup], yerr = averDataList[indexMeas][:,2*numParamPerGroup], fmt='o', label = measInfo[0], markersize = plotMarkSize, capsize = plotCapSize)
    if len(x0) > 2 and indexInterest >= 7 and indexInterest <= 9:   # Flux-related parameters: fitting an exponential function to data
        lines = axMean[1].errorbar(x0, y2, yerr = averDataList[indexMeas][:,indexInterest+numParamPerGroup], fmt='o', label = measInfo[0], markersize = plotMarkSize, capsize = plotCapSize)
        if indexVariable == 0:              # flow scan
            linearArray = np.linspace(np.amin(x0), np.amax(x0), 100)
            if np.argmin(y2) == 0:          # increasing output
                polyCoeffs = np.polyfit(x0, np.log(y2), 1)
                polyFunc = np.poly1d(polyCoeffs)
                fitArray = np.exp(polyFunc(linearArray))
                axMean[1].semilogy(linearArray, fitArray, lines[-2][0].get_color())    # Use the color of the most recently plotted line
#                axMean[1].plot(linearArray, fitArray, lines[-2][0].get_color())    # Use the color of the most recently plotted line
                print('Doubles for every gas flow {:.2f} '.format(np.log(2)/polyCoeffs[0]) + unitArray[indexVariable].to_string())
            if doesOutput:
                Data4Combo[indexMeas][1][:, :1 + numLangmuirFields] = np.vstack( (averDataList[indexMeas][:, paramStrList.index('Gas flow')],
                                                                       averDataList[indexMeas][:, paramStrList.index('Pressure')],
                                                                       averDataList[indexMeas][:, paramStrList.index('Te')],
                                                                       averDataList[indexMeas][:, paramStrList.index('Te') + numParamPerGroup],
                                                                       averDataList[indexMeas][:, paramStrList.index('Te') + numParamPerGroup],                                                                       
                                                                       averDataList[indexMeas][:, paramStrList.index('Ne')]/averDataList[indexMeas][:, paramStrList.index('Gas density')],
                                                                       averDataList[indexMeas][:, paramStrList.index('Ion flux')],
                                                                       averDataList[indexMeas][:, paramStrList.index('Ion flux') + numParamPerGroup],
                                                                       averDataList[indexMeas][:, paramStrList.index('Ion flux') + numParamPerGroup])
                                                                     ).T
                outputDataFrame = pd.DataFrame(Data4Combo[indexMeas][1][:, 1:], index = Data4Combo[indexMeas][1][:, 0], columns = comboHeaders[1:])
                outputDataFrame.to_excel(writer, sheet_name = str(rootsFreqList[indexMeas]) + 'Hz')
        elif indexKeywordExtraFound == 0:   # position scan
            linearArray = np.linspace(minGaussPosition, maxGaussPosition, 100)
            indexPositionMaxDensity = np.argmax(y2)
            paramsGaussian0 = [np.amax(y2), averDataList[indexMeas][indexPositionMaxDensity,indexVariable], averDataList[indexMeas][indexPositionMaxDensity, indexVariable]/2 ]    # the initial guess for the fitting coefficients (A, mu and sigma)
            indicesFitting = range(np.squeeze(np.where(x0 == minGaussPosition)), np.squeeze(np.where(x0 == maxGaussPosition)))
            GaussianParams, GaussianCovs = curve_fit(Gaussian, x0[indicesFitting], y2[indicesFitting], p0 = paramsGaussian0)
            fitArray = Gaussian(linearArray, *GaussianParams)   # Get the fitting vector
            axMean[1].plot(linearArray, fitArray, lines[-2][0].get_color())  # Use the color of the most recently plotted line
            presArray[indexMeas] = np.mean(averDataList[indexMeas][:, paramStrList.index('Pressure')])
            factorFWHM = 2*np.sqrt(2*np.log(2))
            widthArray[indexMeas] = factorFWHM*np.abs(GaussianParams[2])
            print('Peak of the Gassian ' + "{:.3g}".format(GaussianParams[0]) + ' ' + unitArray[indexInterest].to_string())
            print('Center of the Gassian ' + "{:.1f}".format(GaussianParams[1]) + ' ' + unitArray[indexInterest].to_string())
            print('FWHM of the Gassian {:.2f} ± {:.2f}'.format(widthArray[indexMeas], factorFWHM*np.sqrt(np.diag(GaussianCovs)[2])) + ' ' + unitArray[indexVariable].to_string())
    else:
        axMean[1].plot(x0, y2, 'o', label = measInfo[0], markersize = plotMarkSize)
#        if paramStrList[indexInterest] == 'Sheath thickness': # sheath thickness is concerned
#            meanFreePathNN = (1/(averDataList[indexMeas][:, paramStrList.index('Gas density')]*densityUnit)/crossSectionNN).to(thicknessUnit)  # Collision with gas molecules
#            crossSectionHardSph = math.pi*(2*gasDict.get(settingFile[0:2],{}).get('radius',-1))**2
#            axMean[1].plot(x0, meanFreePathNN, 'o', markersize = plotMarkSize, label = 'Mean free path gas molecules')

if 'writer' in locals():
    writer.save()
    writer.close()
if len(np.where(widthArray > 0)[0]) > 2:
    figWidth, axWidth = plt.subplots(1, figsize=(plotFigSizeX, plotFigSizeY))
    lines = axWidth.plot(presArray, widthArray, 'o', markersize = plotMarkSize, label = 'Data')
    secondRootParams0 = [(widthArray[-1] - widthArray[0])/(presArray[-1] - presArray[0]), widthArray[0]]    # the initial guess for the fitting coefficients
    secondRootParams,_ = curve_fit(SecondRoot, presArray, widthArray, p0 = secondRootParams0)
    linearArray = np.linspace(presArray[-1], presArray[0], 100)
    fourthRootArray = SecondRoot(linearArray, *secondRootParams)
    axWidth.plot(linearArray, fourthRootArray, lines[-1].get_color(), label = 'Fourth Root of pressure')
    FigureDecoration(axWidth, 'Pressure (' + unitArray[paramStrList.index('Pressure')].to_string('latex') + ')', 'Plasma width (mm)', '')

FigureDecoration(axMean[0], '', 'Temperature ['+ unitArray[numParamPerGroup].to_string('latex') + ']', gasDict.get(settingFile[0:2],'None').get('name','Unknown gas') + ' Plasma Parameters nano-PSI at '+'{:.1f}'.format(list(set(averDataList[indexMeas][:, indexConstant]))[0]) + ' ' + unitArray[indexConstant].to_string('latex'))
FigureDecoration(axMean[1], 'Bias voltage' if indexKeywordExtraFound == 1 else paramStrList[indexVariable] + ' (' + unitArray[indexVariable].to_string('latex') + ')', paramStrList[indexInterest] + ' (' + unitArray[indexInterest].to_string('latex') + ')', '')
plt.tight_layout()
