# -*- coding: utf-8 -*-
"""
Created on Mon May 10 21:32:38 2021

@author: scwang
"""
from astropy import units as u
import astropy.constants as const
import matplotlib.pyplot as plt
from matplotlib.ticker import ScalarFormatter
import numpy as np
import pandas as pd

u.slm = u.def_unit('slm', u.l/u.min)
plt.close("all")

#%% Inputs

freqList = [25, 30, 35, 42, 50]

#%% Constants

dataFile = 'source_combo_database.xlsx'
flowUnit = u.slm
tempUnit = u.eV
fluxUnit = u.m**-2*u.s**-1
presSLM = 1*u.bar
tempSLM = 0*u.deg_C     # Since 1982
diamTarget = 18*u.mm
plotMarkList = ['o', '^', 's', 'D']
plotFigSizeX = 12
plotFigSizeY = 10
plotMarkSize = 10
plotFontSize = 16
plotCapSize = 8

#%% Functions
def FigureDecoration(ax, xStr, yStr, tStr):
    ax.set_xlabel(xStr, fontsize = plotFontSize)
    ax.set_ylabel(yStr, fontsize = plotFontSize)
    ax.set_title(tStr, fontsize = plotFontSize)
    ax.xaxis.set_tick_params(labelsize = plotFontSize)
    ax.yaxis.set_tick_params(labelsize = plotFontSize)
    ax.yaxis.get_offset_text().set_size(plotFontSize)                   # the exponent is an offset computed by matplotlib
    ax.yaxis.set_major_formatter(ScalarFormatter(useMathText=True))     # Use the scientific notation for the offset
    if 'efficiency' in yStr:
        ax.ticklabel_format(axis = "y", style = "sci", scilimits = (0,0))   # Deliberately use the scientific notation for the ticks on the y-axis. Perform this command after get_offset_text() and set_major_formatter()
    if xStr:    # The bottom subplot of the figure
        ax.legend(fontsize = plotFontSize, loc = 'upper left')

#%% Readout

reshapedDataList = []
for freq in freqList:
    excelDataFrame = pd.read_excel(dataFile, sheet_name = str(freq) + 'Hz')
    comboDataMatrix = excelDataFrame.values
    for indexRaw, flowRaw in enumerate(comboDataMatrix[:, 0]):
        comboDataToAppend = np.atleast_2d(comboDataMatrix[indexRaw, 1:])
        if not reshapedDataList:   # The combo list is empty
            reshapedDataList += [[flowRaw, comboDataToAppend]]
        else:
            isFlowFound = False
            for reshapedData in reshapedDataList:
                if reshapedData[0] == flowRaw:
                    isFlowFound = True
                    reshapedData[1] = np.vstack( (reshapedData[1], comboDataToAppend) )
                    break
            if not isFlowFound:
                reshapedDataList += [[flowRaw, comboDataToAppend]]

#%% Visualization
figPres, axPres = plt.subplots(1, figsize=(plotFigSizeX, plotFigSizeY) )
axPres2 = axPres.twinx()
figPerf, axPerf = plt.subplots(3, sharex = True, figsize=(plotFigSizeX, plotFigSizeY) )
figEff, axEff = plt.subplots(2, sharex = True, figsize=(plotFigSizeX, plotFigSizeY) )
areaTarget = np.pi/4*diamTarget**2
colorList = plt.rcParams['axes.prop_cycle'].by_key()['color']   # Default color list
for indexData, reshapedData in enumerate(reshapedDataList):
    sizeDataPerSet = int(reshapedData[1].shape[1]/3)
    axPres.errorbar(reshapedData[1][:, 0],   reshapedData[1][:, 1 +   sizeDataPerSet],   yerr = reshapedData[1][:, 2 +   sizeDataPerSet:4 +   sizeDataPerSet].T,                                           color = colorList[0], fmt=plotMarkList[indexData], label = str(reshapedData[0]) + ' ' + flowUnit.to_string('latex'), markersize = plotMarkSize, capsize = plotCapSize)
    axPres2.errorbar(reshapedData[1][:, 0],  reshapedData[1][:, 1 + 2*sizeDataPerSet],   yerr = reshapedData[1][:, 2 + 2*sizeDataPerSet:4 + 2*sizeDataPerSet].T, color = colorList[1], fmt=plotMarkList[indexData], label = str(reshapedData[0]) + ' ' + flowUnit.to_string('latex'), markersize = plotMarkSize, capsize = plotCapSize)
    axPerf[0].errorbar(reshapedData[1][:, 0],  reshapedData[1][:, 1],                      yerr = reshapedData[1][:, 2]                                          , fmt=plotMarkList[indexData], label = str(reshapedData[0]) + ' ' + flowUnit.to_string('latex'), markersize = plotMarkSize, capsize = plotCapSize)
    axPerf[1].errorbar(reshapedData[1][:, 0],  reshapedData[1][:, 1 + sizeDataPerSet],     yerr = reshapedData[1][:, 2 +   sizeDataPerSet:4 +   sizeDataPerSet].T, fmt=plotMarkList[indexData], label = str(reshapedData[0]) + ' ' + flowUnit.to_string('latex'), markersize = plotMarkSize, capsize = plotCapSize)
    axPerf[2].errorbar(reshapedData[1][:, 0],  reshapedData[1][:, 1 + 2*sizeDataPerSet],   yerr = reshapedData[1][:, 2 + 2*sizeDataPerSet:4 + 2*sizeDataPerSet].T, fmt=plotMarkList[indexData], label = str(reshapedData[0]) + ' ' + flowUnit.to_string('latex'), markersize = plotMarkSize, capsize = plotCapSize)
    gasFlowRate = (reshapedData[0]*flowUnit*presSLM/const.R/tempSLM.to(u.K, equivalencies = u.temperature())*const.N_A).si
    ionEffMatrix = (reshapedData[1][:, 1 +    sizeDataPerSet :4 +   sizeDataPerSet]*fluxUnit*areaTarget/gasFlowRate).si.value
    atomEffMatrix = (reshapedData[1][:, 1 + 2*sizeDataPerSet :4 + 2*sizeDataPerSet]*fluxUnit*areaTarget/gasFlowRate).si.value
    axEff[0].errorbar(reshapedData[1][:, 0],  ionEffMatrix[:, 0], yerr =  ionEffMatrix[:, 1:3].T, fmt=plotMarkList[indexData], label = str(reshapedData[0]) + ' ' + flowUnit.to_string('latex'), markersize = plotMarkSize, capsize = plotCapSize)
    axEff[1].errorbar(reshapedData[1][:, 0], atomEffMatrix[:, 0], yerr = atomEffMatrix[:, 1:3].T, fmt=plotMarkList[indexData], label = str(reshapedData[0]) + ' ' + flowUnit.to_string('latex'), markersize = plotMarkSize, capsize = plotCapSize)
    
comboHeaderMatrix = excelDataFrame.columns
FigureDecoration(axPres, comboHeaderMatrix[1], comboHeaderMatrix[2] + ' (' + fluxUnit.to_string('latex') + ')', '')
FigureDecoration(axPres2, comboHeaderMatrix[1], comboHeaderMatrix[2 + sizeDataPerSet] + ' (' + fluxUnit.to_string('latex') + ')', '')
FigureDecoration(axPerf[0],                   '', comboHeaderMatrix[2]                      + ' (' + tempUnit.to_string('latex') + ')', '')
FigureDecoration(axPerf[1],                   '', comboHeaderMatrix[2 +   sizeDataPerSet] + ' (' + fluxUnit.to_string('latex') + ')', '')
FigureDecoration(axPerf[2], comboHeaderMatrix[1], comboHeaderMatrix[2 + 2*sizeDataPerSet] + ' (' + fluxUnit.to_string('latex') + ')', '')
FigureDecoration(axEff[0],                   '', 'Gas efficiency for ions', '')
FigureDecoration(axEff[1], comboHeaderMatrix[1], 'Gas efficiency for atoms', '')
